@@ -7,10 +7,12 @@
 
  Module to provide class hierachy to simplify access to the BloxOne APIs
 
- Date Last Updated: 20200907
+ Date Last Updated: 20210308
 
  Todo:
 
+    api_key format verification upon inifile read.
+
  Copyright (c) 2020 Chris Marrison / Infoblox
 
  Redistribution and use in source and binary forms,
@@ -42,16 +44,37 @@
 import logging
 import configparser
 import requests
+import os
+import re
 import json
 
 # ** Global Vars **
-__version__ = '0.6.1'
+__version__ = '0.7.0'
 __author__ = 'Chris Marrison'
 __email__ = 'chris@infoblox.com'
 __doc__ = 'https://python-bloxone.readthedocs.io/en/latest/'
 __license__ = 'BSD'
 
 
+# Custom Exceptions
+class IniFileSectionError(Exception):
+    '''
+    Exception for missing section in ini file
+    '''
+    pass
+
+class IniFileKeyError(Exception):
+    '''
+    Exception for missing key in ini file
+    '''
+    pass
+
+class APIKeyFormatError(Exception):
+    '''
+    Exception for API key format mismatch
+    '''
+    pass
+
 # ** Facilitate ini file for basic configuration including API Key
 
 def read_b1_ini(ini_filename):
@@ -62,40 +85,91 @@ def read_b1_ini(ini_filename):
         ini_filename (str): name of inifile
 
     Returns:
-        config :(dict): Dictionary of BloxOne configuration elements
+        config (dict): Dictionary of BloxOne configuration elements
+
+    Raises:
+        IniFileSectionError
+        IniFileKeyError
+        APIKeyFormatError
+        FileNotFoundError
 
     '''
     # Local Variables
     cfg = configparser.ConfigParser()
     config = {}
     ini_keys = ['url', 'api_version', 'api_key']
+ 
+    # Check for inifile and raise exception if not found
+    if os.path.isfile(ini_filename):
+        # Attempt to read api_key from ini file
+        try:
+            cfg.read(ini_filename)
+        except configparser.Error as err:
+            logging.error(err)
 
-    # Attempt to read api_key from ini file
-    try:
-        cfg.read(ini_filename)
-    except configparser.Error as err:
-        logging.error(err)
+        # Look for BloxOne section
+        if 'BloxOne' in cfg:
+            for key in ini_keys:
+                # Check for key in BloxOne section
+                if key in cfg['BloxOne']:
+                    config[key] = cfg['BloxOne'][key].strip("'\"")
+                    logging.debug('Key {} found in {}: {}'
+                                 .format(key, ini_filename, config[key]))
+                else:
+                    logging.error('Key {} not found in BloxOne section.'
+                                 .format(key))
+                    raise IniFileKeyError('Key "' + key + '" not found within' 
+                        '[BloxOne] section of ini file {}'.format(ini_filename))
+                    
+        else:
+            logging.error('No BloxOne Section in config file: {}'
+                         .format(ini_filename))
+            raise IniFileSectionError('No [BloxOne] section found in ini file {}'
+                                     .format(ini_filename))
+        
+        # Verify format of API Key
+        if verify_api_key(config['api_key']):
+            logging.debug('API Key passed format verification')
+        else:
+            logging.debug('API Key {} failed format verification'
+                          .format(config['api_key']))
+            raise APIKeyFormatError('API Key {} failed format verification'
+                                    .format(config['api_key']))
 
-    # Look for TIDE section
-    if 'BloxOne' in cfg:
-        for key in ini_keys:
-            # Check for key in BloxOne section
-            if key in cfg['BloxOne']:
-                config[key] = cfg['BloxOne'][key].strip("'\"")
-                logging.debug('Key {} found in {}: {}'.format(key, ini_filename, config[key]))
-            else:
-                logging.warn('Key {} not found in BloxOne section.'.format(key))
-                config[key] = ''
     else:
-        logging.warn('No BloxOne Section in config file: {}'.format(ini_filename))
-        config['api_key'] = ''
+        raise FileNotFoundError('ini file "{}" not found.'.format(ini_filename))
 
     return config
 
 
+def verify_api_key(apikey):
+    '''
+    Verify format of API Key
+    
+    Parameters:
+       apikey (str): api key
+
+    Returns:
+        bool: True is apikey passes format validation
+    '''
+    if re.fullmatch('[a-z0-9]{32}|[a-z0-9]{64}', apikey, re.IGNORECASE):
+        status = True
+    else:
+        status = False
+
+    return status
+
+
 class b1:
     '''
     Parent Class to simplify access to the BloxOne APIs for subclasses
+    Can also be used to genericallly access the API
+
+    Raises:
+        IniFileSectionError
+        IniFileKeyError
+        APIKeyFormatError
+        FileNotFoundError
     '''
 
     def __init__(self, cfg_file='config.ini'):
@@ -120,9 +194,16 @@ class b1:
         # Create base URLs
         self.base_url = self.cfg['url']
         self.api_version = self.cfg['api_version']
+
+        self.anycast_url = self.base_url + '/api/anycast/' + self.cfg['api_version']
+        self.authn_url = self.base_url + '/api/authn/' + self.cfg['api_version']
+        self.bootstrap_url = self.base_url + '/api/atlas-bootstrap-app/' + self.cfg['api_version']
+        self.cdc_url = self.base_url + '/api/cdc-flow/' + self.api_version
         self.ddi_url = self.base_url + '/api/ddi/' + self.api_version
         self.host_url = self.base_url + '/api/host_app/' + self.cfg['api_version']
-        self.anycast_url = self.base_url + '/api/anycast/' + self.cfg['api_version']
+        self.sw_url = self.base_url + '/api/upgrade_policy/' + self.cfg['api_version']
+        self.ztp_url = self.base_url + '/atlas-host-activation/' + self.cfg['api_version']
+        
         self.tdc_url = self.base_url + '/api/atcfw/' + self.cfg['api_version']
         self.tdep_url = self.base_url + '/api/atcep/' + self.cfg['api_version']
         self.tddfp_url = self.base_url + '/api/atcdfp/' + self.cfg['api_version']
@@ -131,6 +212,7 @@ class b1:
         self.dossier_url = self.base_url + '/tide/api/services/intel/lookup'
         self.threat_enrichment_url = self.base_url + '/tide/threat-enrichment'
 
+
         # List of successful return codes
         self.return_codes_ok = [200, 201, 204]
 
@@ -268,26 +350,20 @@ class b1:
         return url
 
 
-class b1platform(b1):
-    '''
-    Class to simplify access to the BloxOne Platform APIs
-    '''
-
-    def get(self, objpath, id="", action="", **params):
+    # Public Generic Methods
+    def get(self, url, id="", action="", **params):
         '''
-        Generic get object wrapper for platform calls
+        Generic get object wrapper 
 
         Parameters:
-            objpath (str):  Swagger object path
+            url (str):      Full URL
             id (str):       Optional Object ID
             action (str):   Optional object action, e.g. "nextavailableip"
-
+        
         Returns:
             response object: Requests response object
         '''
-
         # Build url
-        url = self.host_url + objpath
         url = self._use_obj_id(url,id=id)
         url = self._add_params(url, **params)
         logging.debug("URL: {}".format(url))
@@ -296,20 +372,19 @@ class b1platform(b1):
 
         return response
 
-        
-    def create(self, objpath, body=""):
+
+    def create(self, url, body=""):
         '''
-        Generic create object wrapper for platform objects
+        Generic create object wrapper 
 
         Parameters:
-            objpath (str):  Swagger object path
-            body (str):     JSON formatted data payload
+            url (str):  Full URL
+            body (str): JSON formatted data payload
 
         Returns:
             response object: Requests response object
         '''
         # Build url
-        url = self.host_url + objpath
         logging.debug("URL: {}".format(url))
 
         # Make API Call
@@ -318,20 +393,19 @@ class b1platform(b1):
         return response
 
 
-    def delete(self, objpath, id=""):
+    def delete(self, url, id=""):
         '''
-        Generic delete object wrapper for platform objects
+        Generic delete object wrapper
 
         Parameters:
-            objpath (str):  Swagger object path
-            id (str):       Object id to delete
+            url (str):  Full URL
+            id (str):   Object id to delete
 
         Returns:
             response object: Requests response object
         '''
         # Build url
-        url = self.host_url + objpath
-        url = self._use_obj_id(url, id=id)
+        url = self._use_obj_id(url,id=id)
         logging.debug("URL: {}".format(url))
 
         # Make API Call
@@ -340,19 +414,18 @@ class b1platform(b1):
         return response
 
 
-    def update(self, objpath, id="", body=""):
+    def update(self, url, id="", body=""):
         '''
-        Generic create object wrapper for ddi objects
+        Generic create object wrapper 
 
         Parameters:
-            objpath (str):  Swagger object path
-            body (str):     JSON formatted data payload
+            url (str):  Full URL
+            body (str): JSON formatted data payload
 
         Returns:
             response object: Requests response object
         '''
-        # Build url
-        url = self.host_url + objpath
+        # Build url if needed
         url = self._use_obj_id(url, id=id)
         logging.debug("URL: {}".format(url))
 
@@ -362,124 +435,22 @@ class b1platform(b1):
         return response
 
 
-    def get_tags(self, objpath, id=""):
+    def replace(self, url, id="", body=""):
         '''
-        Get tags for an object id
+        Generic create object wrapper 
 
         Parameters:
-            objpath (str):  Swagger object path
-
-            id (str): id of object
-
-        Returns:
-            tags (dict): Dictionary of current tags
-                         or empty dict if none
-        
-        .. todo::
-            * make generic, however, this requires the below
-            * Lookup dictionary of 'required fields' per object type
-        '''
-        tags = {}
-        response = self.get(objpath, id=id, _fields="tags")
-        if response.status_code in self.return_codes_ok:
-            tags = json.loads(response.text)
-            tags = tags['result']
-        else:
-            tags = {}
-        
-        return tags
-
-
-    # *** Platform API Requests *** 
-
-    def on_prem_hosts(self, **params):
-        '''
-        Method to retrieve On Prem Hosts
-        (undocumented)
-
-        Parameters:
-            **params (dict): Generic API parameters
+            url (str):  Full URL
+            body (str): JSON formatted data payload
 
         Returns:
             response object: Requests response object
         '''
-
-        # Call BloxOne API
-        response = self.get('/on_prem_hosts', **params)
-
-        # Return response object
-        return response 
-
-
-    def oph_add_tag(self, id="", tagname="", tagvalue=""):
-        '''
-        Method to add a tag to an existing On Prem Host
-
-        Parameters:
-            objpath (str):  Swagger object path
-            tagname (str): Name of tag to add
-            tagvalue (str): Value to associate
-
-        Returns:
-            response object: Requests response object
-        '''
-        # tags = self.get_tags('/on_prem_hosts', id=id)
-        response = self.get('/on_prem_hosts', id=id, _fields="display_name,tags")
-        if response.status_code in self.return_codes_ok:
-            data = response.json()['result']
-        else:
-            data = {}
-        logging.debug("Existing tags: {}".format(data))
-        # Add new tag to data
-        if tagname:
-            data['tags'].update({tagname: tagvalue})
-            logging.debug("New tags: {}".format(data))
-        # Update object
-        response = self.update('/on_prem_hosts', id=id, body=json.dumps(data))
-
-        return response
-
-
-    def oph_delete_tag(self, id="", tagname=""):
-        '''
-        Method to delete a tag from an existing On Prem Host
-
-        Parameters:
-            objpath (str):  Swagger object path
-            tagname (str): Name of tag to add
-
-        Returns:
-            response object: Requests response object
-        '''
-        # tags = self.get_tags('/on_prem_hosts', id=id)
-        response = self.get('/on_prem_hosts', id=id, _fields="display_name,tags")
-        if response.status_code in self.return_codes_ok:
-            data = response.json()['result']
-            logging.debug("Existing tags: {}".format(data))
-            # Delete tag from data
-            if tagname in data['tags'].keys():
-                data['tags'].pop(tagname, True)
-                print(json.dumps(data))
-                logging.debug("New tags: {}".format(data))
-                # Update object
-                response = self.update('/on_prem_hosts', id=id, body=json.dumps(data))
-
-        return response
-
-
-    def auditlog(self, **params):
-        '''
-        Get the audit log
-
-        Parameters:
-            **params (dict): Generic API parameters
-        Returns:
-            response object: Requests response object
-        '''
-        url = self.base_url + '/api/auditlog/' + self.api_version +'/logs'
-        url = self._add_params(url, **params)
+        # Build url
+        url = self._use_obj_id(url, id=id)
         logging.debug("URL: {}".format(url))
 
-        response = self._apiget(url)
+        # Make API Call
+        response = self._apipatch(url, body)
 
-        return response
+        return response
\ No newline at end of file
